<?php
/**
* @brief  Library for processing, request and receive data with JapanCase API.
* @date   2015-03-21 20:08:00
* @license http://opensource.org/licenses/GPL-3.0 GNU Public License
* @company: JapanCase.com
* @author: Daniel Omine <omine@japancase.com>
* @updated: 2015-03-27 10:59:00
*/

include_once(dirname(__FILE__).'/JcCurl.php');

class JapanCase
{

    const API_URL_BASE = 'https://pay.japancase.com';
    const API_URL_PORT = '443';
    //const API_URL_BASE = 'http://dev-pay.japancase.com';
    //const API_URL_PORT = '80';

    /**
    Communicates with JapanCase API to validate the store id, url and email
    */
    public function ValidateCode($contract_code, $tel, $store_name, $store_url, $store_email)
    {
        /**
        Parameters to be sent
        */
        $data = array(
            'p'                    => 'Provider/ValidateCode',
            'jc_service_provider'  => 'Epsilon',
            'jc_action_url'        => 'get_client_info',
            'contract_code'        => $contract_code,
            'tel_4'                => substr($this->numbersOnly($tel), -4),
            'jc_store_name'        => $store_name,
            'jc_store_url'         => $store_url,
            'jc_store_email'       => $store_email
        );

        /** Debug purposes */
        //print_r($data); exit;

        $rs = $this -> GatewayRequest($data);

        /** Debug purposes */
        //print_r($rs); exit;

        return $rs;
    }

    /*
    Send payment request (first transaction)
    */
    public function transactionSend($data)
    {
        /** Debug purposes */
        //print_r($data); //exit;

        $rs = $this -> GatewayRequest($data);

        /** Debug purposes */
        //print_r($rs->provider); exit;

        return $rs;
    }

    /*
    Resquest payment sale to an existing provisory payment
    */
    public function AmountChange($jc_service_provider, $contract_code, $jc_store_id, $order_number, $jc_debug_urls, $user_id, $item_code, $new_item_price)
    {
        /**
        Parameters to be sent
        */
        $data = array(
            'p'                    => 'Transaction/AmountChange',
            'jc_service_provider'  => $jc_service_provider,
            'jc_action_url'        => Epsilon::EPSILON_API_URL_CHANGE_AMOUNT,
            'contract_code'        => $contract_code,
            'jc_store_id'          => $jc_store_id,
            'order_number'         => $order_number,
            'jc_debug_urls'        => $jc_debug_urls,
            'user_id'              => $user_id,
            'item_code'            => $item_code,
            'new_item_price'       => $new_item_price,
            'mission_code'         => Epsilon::DEFAULTS_MISSION_CODE,
        );

        /** Debug purposes */
        //print_r($data); exit;
        //return $data;

        $rs = $this->GatewayRequest($data);

        /** Debug purposes */
        //print_r($rs); exit;

        return $rs;
    }

    /*
    Resquest payment sale to an existing provisory payment
    */
    public function RealSale($jc_service_provider, $contract_code, $jc_store_id, $order_number, $jc_debug_urls)
    {
        /**
        Parameters to be sent
        */
        $data = array(
            'p'                    => 'Transaction/RealSale',
            'jc_service_provider'  => $jc_service_provider,
            'jc_action_url'        => Epsilon::EPSILON_API_URL_SALES_PAYMENT,
            'contract_code'        => $contract_code,
            'jc_store_id'          => $jc_store_id,
            'order_number'         => $order_number,
            'jc_debug_urls'        => $jc_debug_urls,
        );

        /** Debug purposes */
        //print_r($data); exit;
        //return $data;

        $rs = $this -> GatewayRequest($data);

        /** Debug purposes */
        //print_r($rs); exit;

        return $rs;
    }

    /*
    Resquest payment cancelation to an existing provisory payment
    */
    public function Cancel($jc_service_provider, $contract_code, $jc_store_id, $order_number, $jc_debug_urls)
    {
        /**
        Parameters to be sent
        */
        $data = array(
            'p'                    => 'Transaction/Cancel',
            'jc_service_provider'  => $jc_service_provider,
            'jc_action_url'        => Epsilon::EPSILON_API_URL_CANCEL,
            'contract_code'        => $contract_code,
            'jc_store_id'          => $jc_store_id,
            'order_number'         => $order_number,
            'jc_debug_urls'        => $jc_debug_urls,
        );

        /** Debug purposes */
        //print_r($data); exit;
        //return $data;

        $rs = $this -> GatewayRequest($data);

        /** Debug purposes */
        //print_r($rs); exit;

        return $rs;
    }

    private function GatewayRequest($data)
    {
        /** 
        Object to hold the results.
        */
        $rs = (object) array(); // new stdClass();

        $rs -> data             = null;
        $provider_response      = null;
        $rs -> error            = false;


        $arr['URL']             = self::API_URL_BASE;
        $arr['PORT']            = self::API_URL_PORT;
        //$arr['HEADER']          = true;
        $arr['MAXREDIRS']       = 3;
        $arr['REFERER']         = _PS_BASE_URL_;
        $arr['USER_AGENT']      = 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.1; Trident/6.0)';
        $arr['USERPWD']         = false;
        $arr['HTTPAUTH']        = false; // HTTP_AUTH_NTLM, CURLAUTH_BASIC
        $arr['METHOD']          = 'POST';
        $arr['TIMEOUT']         = 15;
        $arr['CONNECTTIMEOUT']  = 15;
        $arr['ENCODE_DATA']     = 2;
        $arr['SSL_VERIFYHOST']  = 2; // false
        $arr['SSL_VERIFYPEER']  = true; // 2
        $arr['SSLVERSION']      = 6; //CURL_SSLVERSION_TLSv1_2
        $arr['CAINFO']          = dirname(__FILE__).DIRECTORY_SEPARATOR.'cacert.pem';

        /**
        Calling the CURL instance.
        */
        $c = JcCurl::Set($arr);

        /** 
        Data to be sent (form fields).
        */
        $c::$fields = $data;

        /** Clear used objects */
        unset($data);

        /** Debug purposes */
        //echo __FILE__.PHP_EOL; print_r($c::$fields); exit;

        /** 
        Requesting the data.
        */
        $c::Request();

        /** Debug purposes */
        //if (isset($c::$fields['acs_pares'])){
        //echo __FILE__.'<br />'.PHP_EOL; print_r($c::$Error); echo '<br />'.PHP_EOL.$c::$Result; exit;
        //}

        if (!$c::$Error) {

            /**
            Remove UTF8 bom
            */
            $c::$Result = str_replace("\xEF\xBB\xBF" ,'', $c::$Result);

            /** 
            Converting the json result to array.
            */
            $provider_response = json_decode(trim($c::$Result), true);
            //$provider_response = json_decode(str_replace("\\", "\\\\", trim($c::$Result)), true);

            /** Debug purposes */
            //echo 'json_last_error: '.json_last_error().PHP_EOL;
            //print_r($provider_response); exit;

            if (json_last_error() > 0) {
                $rs->error['json_error_code'] = json_last_error();
            }

            /** Debug purposes */
            //print_r($rs); exit;

            if (!$rs->error) {
                (($provider_response['error'] !== false && !empty($provider_response['error']))? $rs -> error = $provider_response['error'] : '');
            }

            if (!$rs->error) {

                switch ($c::$fields['p']) {
                    default:
                        $rs -> data = $provider_response['data'];
                        $rs -> provider = $provider_response['provider'];
                    break;
                    case 'Transaction/Send':
                    case 'Transaction/RealSale':
                    case 'Transaction/AmountChange':
                    case 'Transaction/Cancel':
                        $rs -> provider = $provider_response['provider_data'];
                    break;
                }

                /** Debug purposes */
                //print_r($rs -> data); exit;

            } else {

                if (isset($provider_response['error_detail'])) {
                    $rs -> error_detail = $provider_response['error_detail'];
                }

            }

            /** Clear used objects */
            unset($provider_response);

        } else {
            $rs -> error['curl'] = $c::$Error.':'.$c::$ErrorMsg;
        }

        /** Clear used objects */
        unset($c, $arr);

        /** Debug purposes */
        //print_r($rs); exit;

        /** Returns results */
        return $rs;

    }

    public function numbersOnly($str, $float = false, $ignore_chars = false)
    {
        if (!is_array($str) && !empty($str)) {
            $r = '';
            if ($float) {
                $r   = '.';
                $str = str_replace(',', $r, $str);
            }
            if ($ignore_chars) {
                $r .= $ignore_chars;
            }
            return preg_replace('#[^0-9'.$r.']#', '', mb_convert_kana($str, 'n'));
        }
        return '';
    }

    public function validEmailFormat( $str )
    {
        $rule  = '/^([0-9,a-z,A-Z,_,-,.]+)([.,_,-]([0-9,a-z,A-Z,_,-,.]+))';
        $rule .= '*[@]([0-9,a-z,A-Z]+)([.,-]([0-9,a-z,A-Z]+))';
        $rule .= '*[.]([0-9,a-z,A-Z]){2}([0-9,a-z,A-Z])?$/';

        //ereg
        if(preg_match($rule, $str)) {
            return true;
        } else {
            return false;
        }
    }
}
